// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/ZFinder.hh"
#include "Rivet/Projections/FastJets.hh"

namespace Rivet {

  using namespace std;
  //using namespace Cuts;
  
  /// @brief Study of forward Z + jet production at 7 TeV at LHCb
  /// @author W. Barter, A. Bursche, M. Sirendi (Rivet implementation)
  class LHCB_2014_I1262703 : public Analysis {
  public:

    /// Default constructor
    LHCB_2014_I1262703()
      : Analysis("LHCB_2014_I1262703")
    {
    }

    /// @name Analysis methods
    //@{
    
    void init() {
      FinalState fs;
      Cut mycut = (Cuts::eta >= 2.0) & (Cuts::eta <= 4.5) & (Cuts::pT > 20.*GeV);

      ZFinder zfinder(fs, mycut, PID::MUON, 60*GeV, 120*GeV, 0., ZFinder::NOCLUSTER);
      addProjection(zfinder, "ZFinder");
      FastJets jetpro(zfinder.remainingFinalState(), FastJets::ANTIKT, 0.5);
      addProjection(jetpro, "Jets");

      _h_jet_pT = bookHisto1D(3, 1, 1);
      
      _h_jet_eta20 = bookHisto1D(4, 1, 1);
      _h_jet_eta10 = bookHisto1D(4, 1, 2);
      
      _h_Z_y20 = bookHisto1D(5, 1, 1);
      _h_Z_y10 = bookHisto1D(5, 1, 2);
      
      _h_Z_pT20 = bookHisto1D(6, 1, 1);
      _h_Z_pT10 = bookHisto1D(6, 1, 2);
      
      _h_dphi20 = bookHisto1D(7, 1, 1);
      _h_dphi10 = bookHisto1D(7, 1, 2);
      
      _h_dy20 = bookHisto1D(8, 1, 1);
      _h_dy10 = bookHisto1D(8, 1, 2);
    }

    /// Do the analysis
    void analyze(const Event & e) {
      const ZFinder& zfinder = applyProjection<ZFinder>(e, "ZFinder");
      if (zfinder.bosons().size() != 1) vetoEvent;
      
      const ParticleVector& leptons = zfinder.constituents();

      Cut jetSelector = (Cuts::eta >= 2.0) & (Cuts::eta <= 4.5) & (Cuts::pT > 10.*GeV);
      const Jets& jets = applyProjection<FastJets>(e, "Jets").jetsByPt(jetSelector);
      
      // Clean the jets against the lepton candidates, as in the paper, with a deltaR cut of 0.4
      vector<const Jet*> cleanedJets;
      for (size_t i = 0; i < jets.size(); i++) {
        bool isolated = true;
        for (size_t j = 0; j < 2; j++) {
          if (deltaR(leptons[j], jets[i]) < 0.4) {
            isolated = false;
            break;
          }
        }
        if (isolated) cleanedJets.push_back(&jets[i]);
      }

      // Require at least 1 jet
      const unsigned int Njets = cleanedJets.size();
      if (Njets < 1) vetoEvent;

      // 20 GeV jet pT threshold
      bool above20 = false;
      if (cleanedJets[0]->pT()/GeV > 20.) above20 = true;
      
      const double weight = e.weight();
      
      _h_jet_pT->fill(cleanedJets[0]->pT()/GeV, weight);
      if(above20) _h_jet_eta20->fill(cleanedJets[0]->eta(), weight);
      _h_jet_eta10->fill(cleanedJets[0]->eta(), weight);
      if(above20) _h_Z_y20->fill(zfinder.boson().rap(), weight);
      _h_Z_y10->fill(zfinder.boson().rap(), weight);
      if(above20) _h_Z_pT20->fill(zfinder.boson().pT()/GeV, weight);
      _h_Z_pT10->fill(zfinder.boson().pT()/GeV, weight);
      double delta = deltaPhi(zfinder.boson(), (*cleanedJets[0]));
      if(above20) _h_dphi20->fill(delta, weight);
      _h_dphi10->fill(delta, weight);
      if(above20) _h_dy20->fill(zfinder.boson().rap() - cleanedJets[0]->rap(), weight);
      _h_dy10->fill(zfinder.boson().rap() - cleanedJets[0]->rap(), weight);
    }


    /// Finalize
    void finalize() {
      normalize(_h_jet_pT);
      normalize(_h_jet_eta20);
      normalize(_h_jet_eta10);
      normalize(_h_Z_y20);
      normalize(_h_Z_y10);
      normalize(_h_Z_pT20);
      normalize(_h_Z_pT10);
      normalize(_h_dphi20);
      normalize(_h_dphi10);
      normalize(_h_dy20);
      normalize(_h_dy10);
    }

    //@}

  private:

    /// @name Histograms
    //@{
    Histo1DPtr _h_jet_pT;
    Histo1DPtr _h_jet_eta20;
    Histo1DPtr _h_jet_eta10;
    Histo1DPtr _h_Z_y20;
    Histo1DPtr _h_Z_y10;
    Histo1DPtr _h_Z_pT20;
    Histo1DPtr _h_Z_pT10;
    Histo1DPtr _h_dphi20;
    Histo1DPtr _h_dphi10;
    Histo1DPtr _h_dy20;
    Histo1DPtr _h_dy10;
    //@}

  };

  DECLARE_RIVET_PLUGIN(LHCB_2014_I1262703);
}
